using System;
using System.Configuration;
using System.Net;
using System.Security.Principal;
using System.ServiceProcess;
using System.Text;
using System.Threading;
using ITHit.WebDAV.Logger;

namespace WebDAVServer.FileSystemListenerService
{
    partial class WebDAVService : ServiceBase
    {
        private Thread thread;
        private bool listening = false;

        public WebDAVService()
        {
            InitializeComponent();
        }

        protected override void OnStart(string[] args)
        {
            listening = true;
            thread = new Thread(new ThreadStart(ThreadProc));
            thread.Start();
        }

        protected override void OnStop()
        {
            listening = false;
            thread.Abort();
            thread.Join();
        }

        private void ThreadProc()
        {
            string uriPrefix = ConfigurationManager.AppSettings["ListenerPrefix"];
            HttpListener listener = new HttpListener();
            listener.Prefixes.Add(uriPrefix);
            //listener.AuthenticationSchemes = AuthenticationSchemes.IntegratedWindowsAuthentication;
            listener.Start();
            
            FileLogger.Level = LogLevel.Debug;

            while (listening)
            {
                IAsyncResult result = listener.BeginGetContext(new AsyncCallback(ListenerCallback), listener);
                result.AsyncWaitHandle.WaitOne();
            }
        }

        public static void ListenerCallback(IAsyncResult result)
        {
            HttpListener listener = (HttpListener)result.AsyncState;
            WDEngine engine = new WDEngine();
            HttpListenerContext context = listener.EndGetContext(result);
            WDRequest request = new WDRequest(context.Request, context.User);
            WDResponse response = new WDResponse(context.Response);

            //WindowsIdentity identity = (WindowsIdentity)context.User.Identity;
            //WindowsImpersonationContext wic = identity.Impersonate(); // Run on behalf of the client
            engine.Run(request, response);
            //wic.Undo();

            if (response.StatusCode == 401)
            {
                byte[] message = new UTF8Encoding().GetBytes("Access denied");
                context.Response.OutputStream.Write(message, 0, message.Length);
            }

            try
            {
                context.Response.Close();
            }
            catch
            {
                // client closed connection before the content was sent
            }
        }
    }
}
