using System;
using System.IO;
using System.Data;
using System.Data.SqlClient;
using ITHit.WebDAV.Server;
using ITHit.WebDAV.Server.Response;

namespace WebDAVServer.SqlStorage
{
	public class LockNull : HierarchyItem, ILockNull
	{
        public LockNull(int id, string name, string path, DateTime created, DateTime modified)
            :base(id, name, path, created, modified)
        {
        }

		public WebDAVResponse ConvertToResource(Stream content, string contentType)
		{
			if(!ClientHasToken)
				return new LockedResponse();

			SqlConnection conn = new SqlConnection(connStr);
			SqlTransaction trans = null;
			conn.Open();
			try
			{
				trans = conn.BeginTransaction();
				CreateItemFromLockNull(trans, ID, ItemType.Resource);
				Resource newResource = new Resource(ID, Name, Path, Created, Modified);
				newResource.WriteContent(trans, content, contentType);
				trans.Commit();
			}
			catch(Exception ex)
			{
				if(trans != null) trans.Rollback();
				throw ex;
			}
			finally
			{
				conn.Close();
			}
			return new CreatedResponse();
		}

		public WebDAVResponse ConvertToFolder()
		{
			if(!ClientHasToken)
				return new LockedResponse();

			SqlConnection conn = new SqlConnection(connStr);
			SqlTransaction trans = null;
			conn.Open();
			try
			{
				trans = conn.BeginTransaction();
				CreateItemFromLockNull(trans, ID, ItemType.Folder);
				trans.Commit();
			}
			catch(Exception ex)
			{
				if(trans != null) trans.Rollback();
				throw ex;
			}
			finally
			{
				conn.Close();
			}

			return new CreatedResponse();
		}

		public override WebDAVResponse CopyTo(IFolder folder, string destName, bool deep)
		{
			return new NotAllowedResponse();
		}

		public override WebDAVResponse MoveTo(IFolder folder, string destName)
		{
			return new NotAllowedResponse();
		}

		public override WebDAVResponse Delete()
		{
			 return new NotAllowedResponse();
		}

		public override WebDAVResponse UpdateProperties(Property[] setProps, Property[] delProps)
		{
			return new NotAllowedResponse();
		}

		private void CreateItemFromLockNull(SqlTransaction trans, int lockNullID, ItemType itemType)
		{
			SqlCommand cmd = trans.Connection.CreateCommand();
			cmd.Transaction = trans;
                
			// set item type
			cmd.CommandText = "UPDATE Repository SET ItemType = @ItemType WHERE ID = @ID";
			cmd.Parameters.Add("@ItemType", SqlDbType.TinyInt).Value = itemType;
			cmd.Parameters.Add("@ID", SqlDbType.Int).Value = lockNullID;
			cmd.ExecuteNonQuery();
                
			UpdateModified(trans);
		}
    }
}
