using System;
using System.Data;
using System.Web;
using System.IO;
using System.Data.SqlClient;
using System.Configuration;
using ITHit.WebDAV.Server;

namespace WebDAVServer.SqlStorage
{
	public class WDEngine : Engine
	{
        private string connStr = ConfigurationSettings.AppSettings["SqlConnectionString"];

        public override IHierarchyItem GetHierarchyItem(string path)
        {
            int itemID = 0;
            ItemType itemType = 0;
            string itemName = "";
            DateTime itemCreated = DateTime.MinValue;
            DateTime itemModified = DateTime.MinValue;
            int id;

            SqlConnection conn = new SqlConnection(connStr);
            SqlCommand cmd;
            SqlDataReader reader = null;
            conn.Open();

            try
            {
                cmd = conn.CreateCommand();
                if(path == "/")
                {
                    // get root folder
                    cmd.CommandText = 
                        "SELECT ID, ItemType, Name, Created, Modified"
                        +" FROM Repository"
                        +" WHERE Parent = 0";
                    reader = cmd.ExecuteReader();
                    if(reader.Read())
                    {
                        itemID = reader.GetInt32(reader.GetOrdinal("ID"));
                        itemType = (ItemType)reader.GetByte(reader.GetOrdinal("ItemType"));
                        itemName = reader.GetString(reader.GetOrdinal("Name"));
                        itemCreated = reader.GetDateTime(reader.GetOrdinal("Created"));
                        itemModified = reader.GetDateTime(reader.GetOrdinal("Modified"));
                    }
                }
                else
                {
                    // find root
                    cmd.CommandText = "SELECT ID FROM Repository WHERE Parent = 0";
                    id = (int)cmd.ExecuteScalar();

                    string[] names = path.Split('/');
                    int last = names.Length-1;
                    while(last > 0 && names[last] == "") last--;
                    // search for item by path
                    cmd.CommandText = 
                        "SELECT ID FROM Repository"
                        +" WHERE Name = @Name"
                        +" AND Parent = @Parent";
                    cmd.Parameters.Add("@Name", SqlDbType.NVarChar, 255);
                    cmd.Parameters.Add("@Parent", SqlDbType.Int, 4);
                    cmd.Prepare();
                    for(int i=0; i<last; i++)
                        if(names[i]!="")
                        {
                            cmd.Parameters["@Name"].Value = names[i];
                            cmd.Parameters["@Parent"].Value = id;
                            id = (int)cmd.ExecuteScalar();
                        }
                    // get item properties
                    cmd.CommandText = 
                        "SELECT ID,ItemType,Name,Created,Modified"
                        +" FROM Repository"
                        +" WHERE Name = @Name"
                        +" AND Parent = @Parent";
                    cmd.Parameters["@Name"].Value = names[last];
                    cmd.Parameters["@Parent"].Value = id;
                    reader = cmd.ExecuteReader();
                    if(reader.Read())
                    {
                        itemID = reader.GetInt32(reader.GetOrdinal("ID"));
                        itemType = (ItemType)reader.GetByte(reader.GetOrdinal("ItemType"));
                        itemName = reader.GetString(reader.GetOrdinal("Name"));
                        itemCreated = reader.GetDateTime(reader.GetOrdinal("Created"));
                        itemModified = reader.GetDateTime(reader.GetOrdinal("Modified"));
                    }
                }
            }
            finally
            {
                if (reader != null) reader.Close();
                conn.Close();
            }

            if(itemID == 0)
                return null;

            switch(itemType)
            {
				case ItemType.Resource:
                    return new Resource(itemID, itemName, path, itemCreated, itemModified);
                case ItemType.Folder:
                    if(!path.EndsWith("/")) path = path+"/";
                    return new Folder(itemID, itemName, path, itemCreated, itemModified);
				case ItemType.LockNull:
					return new LockNull(itemID, itemName, path, itemCreated, itemModified);
				default:
					return null;
            }
        }
        
		// By default WebDAV engine searches for License.lic in the folder where your .dll or .exe file resides.
		// To change this behavior Engine.License is overridden
		public override string License
		{
			get
			{
				TextReader reader = File.OpenText(
					HttpContext.Current.Request.PhysicalApplicationPath + "License.lic");
				string license = reader.ReadToEnd();
				reader.Close();
				return license;
			}
		}
    }
}
